==> linuxconf-tools-1.1.src.tar.gz: linuxconf-tools-1.1/projetx/version1.cc <==
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"

static unsigned *ptselitem=NULL;

#if 0
/*
	Prsente l'information sur la rvision couramment slectionn
	dans la VERSION::select
*/
static PAGE_VAL info_fct (
	long mskwin,
	EVENT *cmd,
	BOUTON_FCT *but,
	void *data)
{
	VERSION_EDIT *vedit = (VERSION_EDIT*)data;
	VERSION_DAT *version = vedit->v;
	const VERSION_ONE *one = version->get(vedit->lookup[*ptselitem]);
	const char *info = one->info;
	if (info == NULL) info = "\n";
	PAGE_EDIT page("version_info");
	PAGE_EDIT spage("");
	spage.setboxtype (WINBOX_NO);
	char verstr[MAXSIZ_REVISION];
	one->rev.format (verstr);
	page_setuptitre (&page,MSGTIT_REVISION,verstr);
	int nbinfo = 0;
	while (*info != '\0'){
		char ligne[80];
		char *pt = ligne;
		if (*info == ';') info++;
		while (*info != '\n' && *info != '\0') *pt++ = *info++;
		if (*info == '\n') info++;
		*pt = '\0';
		int len = strlen(ligne);
		if (len < 30) len = 30;
		page_setupfixestr0 (&spage,-1,ligne,len);
		nbinfo++;
	}
	if (nbinfo > 10) spage.setnbdisp (10);
	spage.coldesign();
	page.setup (-1,&spage,0);
	bouton_setupstd (&page,"cancel");
	page.coldesign();
	page.setspec (but);
	page.edit (mskwin,cmd);
	return PAGE_VAL_NULL;
}

#endif
static void version_add2dia (const VERSION_ONE *one, DIALOG &dia)
{
	char buf1[100],buf2[100];
	one->format(buf1,buf2);
	dia.new_menuitem (buf1,buf2);
}

/*
	Retourne information sur une rvision spcifique.
	Retourne NULL si sel invalide.
*/
PUBLIC VERSION_ONE *VERSION_DAT::get(int sel)
{
	VERSION_ONE *ret = NULL;
	if (sel >= 0 && sel < list.getnb()){
		ret = list.getitem(sel);
	}
	return ret;
}
/*
	Retourne information sur une rvision spcifique.
	Retourne NULL si trouve pas.
*/
PUBLIC VERSION_ONE *VERSION_DAT::get(
	const REVISION *rev,
	int & noitem)	// Contiendra la position de la rvision trouv
					// dans la table (on pourra faire get(noitem) plus tard).
					// ou -1 si trouve pas.
{
	noitem = -1;
	VERSION_ONE *ret = NULL;
	for (int i=0; i<list.getnb(); i++){
		VERSION_ONE *one = list.getitem(i);
		if (one->rev.cmp(rev)==0){
			// Rvision existe
			ret = one;
			noitem = i;
			break;
		}
	}
	return ret;
}
/*
	Retourne information sur une rvision spcifique.
	Retourne NULL si trouve pas.
*/
PUBLIC VERSION_ONE *VERSION_DAT::get(const REVISION *rev)
{
	int noitem;
	return get (rev,noitem);
}
/*
	Retourne le nombre de rvision dans la table.
*/
PUBLIC int VERSION_DAT::getnb()
{
	return list.getnb();
}
/*
	Elimine une rvision de la liste version.dat
	Retourne -1 si trouve pas la rvision.
*/
PUBLIC int VERSION_DAT::delrev(const REVISION *rev)
{
	int ret = -1;
	for (int i=0; i<list.getnb(); i++){
		VERSION_ONE *one = list.getitem(i);
		if (one->rev.cmp(rev)==0){
			// Rvision existe, on l'efface
			list.remove_del (i);
			ret = 0;
			break;
		}
	}
	return ret;
}

#ifdef MISSING
static int near version_setpos (
	PAGE_EDIT *pick,
	int nb)
{
	int ret = -1;
	if (nb > 0){
		/* #Spcification: rvision / slection
			Lorsqu'on slectionne une rvision, la dernire page
			de la liste est toujours prsente en premier.
		*/
		pick->coldesign();
		pick->setpos (nb > 10 ? nb - 10 : 0 ,nb-1);
		ret = 0;
	}else{
		error_showdef (MSGERR_NOVERSION);
	}
	return ret;
}
#endif

/*
	Prparation d'une sous-page pour la slection interactive d'une version.
	Retourne -1 si erreur.
*/
PUBLIC int VERSION_DAT::setselect(
	DIALOG &dia,
	REVISION_TYPE minsel,	// Slection de rvision ou d'une version.
	REVISION_TYPE maxsel,	//
	unsigned *selitem,
	const char *path_check,	// Prsente uniquement les rvision
							// dj dans path_check si != NULL
	int lookup[])			// Correspondance entre le X'ieme item
							// de la PAGE_PICK et un VERSION_ONE
{
	int ret = -1;
	char *path_rev[200];
	int nbprev = -1;
	if (path_check != NULL){
		nbprev = projet_getrevdir (path_check,path_rev,200);
	}
	if (list.getnb() == 0 || nbprev == 0){
		xconf_error (MSG_U(E_NOVERSION,"Aucune rvision slectionnable"));
	}else{
		ptselitem = selitem;
		int nb = 0;
		setintgstatus();
		if (nbprev > 0){
			for (int i=0; i<nbprev; i++){
				char revstr[MAXSIZ_REVISION];
				revision_path2rev(path_rev[i],revstr);
				REVISION rev (revstr);
				int noitem;
				VERSION_ONE *one = get(&rev,noitem);
				if(one != NULL){
					version_add2dia (one,dia);
					lookup[nb] = noitem;
					nb++;
				}
			}
		}else{
			for (int i=0; i<list.getnb(); i++){
				VERSION_ONE *one = list.getitem(i);
				REVISION_TYPE type = one->rev.type();
				if (type >= minsel && type <= maxsel){
					version_add2dia (one,dia);
					lookup[nb] = i;
					nb++;
				}
			}
		}
		#ifdef MISSING
			ret = version_setpos (pick,nb);
		#else
			ret = 0;
		#endif
	}
	return ret;
}
#ifdef MISSING
/*
	Ajoute le bouton qui permet d'obtenir plus d'information sur les versions
*/
PUBLIC void VERSION_DAT::setboutoninfo (PAGE_EDIT *page, VERSION_EDIT *vedit)
{
	bouton_setupfct (page,MSGBUT_INFO,K_NULL,info_fct,vedit);
}
#endif
/*
	Slection interactive d'une rvision d'un projet
	Retourne NULL si abandon ou erreur.
*/
PUBLIC VERSION_ONE *VERSION_DAT::select(
	REVISION_TYPE minsel,	// Slection de rvision ou d'une version.
	REVISION_TYPE maxsel,	//
	const char *path_check)	// Prsente uniquement les rvision
							// dj dans path_check si != NULL
{
	VERSION_ONE *ret = NULL;
	DIALOG_RECORDS dia;
	dia.newf_head ("",MSG_U(H_VERSIONS,"Version\tProg\tDate\tRvision"));
	unsigned selitem = list.getnb()-1;
	int lookup[list.getnb()];
	if (lookup != NULL
		&& setselect (dia,minsel,maxsel,&selitem,path_check,lookup)!=-1){
		#ifdef MISSING
			VERSION_EDIT vedit (this,lookup);
			setboutoninfo (&page,&vedit);
		#endif
		int nof = dia.getnb()-1;
		if (dia.editmenu(MSG_U(T_VERSIONS,"Versions")
			,""
			,help_nil
			,nof,0) == MENU_OK){
			ret = list.getitem(lookup[nof]);
		}
	}
	return ret;
}
/*
	Slection d'une version comme version courante pour usager
	Retourne != 0 si slection valide.
*/
int version_setmain (
	USERINFO *user,
	const char *path_check)	// Prsente uniquement les rvision
							// dj dans path_check si != NULL
{
	int ret = 0;
	VERSION_DAT version(user);
	VERSION_ONE *rev = version.select (REVISION_VERSION
		,REVISION_LIVRE,path_check);
	if (rev != NULL && user->setversion (rev)){
		// Vrifie si l'environnement existe pour l'usager
		ret = 1;
	}
	return ret;
}

/*
	Enregistre toutes les rvisions dans un page (de slection)
*/
PUBLIC void VERSION_DAT::setuppick(DIALOG &dia)
{
	int nb = list.getnb();
	setintgstatus();
	for (int i=0; i<nb; i++){
		VERSION_ONE *one = list.getitem(i);
		version_add2dia (one,dia);
	}
	#ifdef MISSING
		version_setpos (&sp,nb);
	#endif
}

==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/projetx/version1.cc <==
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"
#include <diajava.h>

static unsigned *ptselitem=NULL;

/*
	Prsente l'information sur la rvision couramment slectionn
	dans la VERSION::select
*/
PUBLIC void VERSION_ONE::showinfo()
{
	const char *ptinfo = info;
	if (ptinfo == NULL) ptinfo = "\n";
	DIALOG_TEXTBOX dia;
	while (*ptinfo != '\0'){
		char ligne[800];
		char *pt = ligne;
		if (*info == ';') ptinfo++;
		while (*ptinfo != '\n' && *ptinfo != '\0') *pt++ = *ptinfo++;
		if (*ptinfo == '\n') ptinfo++;
		*pt = '\0';
		dia.newf_text ("",ligne);
	}
	char verstr[MAXSIZ_REVISION],title[100];
	rev.format (verstr);
	snprintf (title,sizeof(title)-1,MSG_U(T_REVTITLE,"Rvision %s"),verstr);
	int nof = 0;
	dia.edit (title,"",help_nil,nof);
}

static void version_add2dia (const VERSION_ONE *one, DIALOG &dia)
{
	char buf1[100],buf2[100];
	one->format(buf1,buf2);
	dia.new_menuitem (buf1,buf2);
}

/*
	Retourne information sur une rvision spcifique.
	Retourne NULL si sel invalide.
*/
PUBLIC VERSION_ONE *VERSION_DAT::get(int sel)
{
	VERSION_ONE *ret = NULL;
	if (sel >= 0 && sel < list.getnb()){
		ret = list.getitem(sel);
	}
	return ret;
}
/*
	Retourne information sur une rvision spcifique.
	Retourne NULL si trouve pas.
*/
PUBLIC VERSION_ONE *VERSION_DAT::get(
	const REVISION *rev,
	int & noitem)	// Contiendra la position de la rvision trouv
					// dans la table (on pourra faire get(noitem) plus tard).
					// ou -1 si trouve pas.
{
	noitem = -1;
	VERSION_ONE *ret = NULL;
	for (int i=0; i<list.getnb(); i++){
		VERSION_ONE *one = list.getitem(i);
		if (one->rev.cmp(rev)==0){
			// Rvision existe
			ret = one;
			noitem = i;
			break;
		}
	}
	return ret;
}
/*
	Retourne information sur une rvision spcifique.
	Retourne NULL si trouve pas.
*/
PUBLIC VERSION_ONE *VERSION_DAT::get(const REVISION *rev)
{
	int noitem;
	return get (rev,noitem);
}
/*
	Retourne le nombre de rvision dans la table.
*/
PUBLIC int VERSION_DAT::getnb()
{
	return list.getnb();
}
/*
	Elimine une rvision de la liste version.dat
	Retourne -1 si trouve pas la rvision.
*/
PUBLIC int VERSION_DAT::delrev(const REVISION *rev)
{
	int ret = -1;
	for (int i=0; i<list.getnb(); i++){
		VERSION_ONE *one = list.getitem(i);
		if (one->rev.cmp(rev)==0){
			// Rvision existe, on l'efface
			list.remove_del (i);
			ret = 0;
			break;
		}
	}
	return ret;
}


/*
	Prparation d'une sous-page pour la slection interactive d'une version.
	Retourne -1 si erreur.
*/
PUBLIC int VERSION_DAT::setselect(
	DIALOG &dia,
	REVISION_TYPE minsel,	// Slection de rvision ou d'une version.
	REVISION_TYPE maxsel,	//
	unsigned *selitem,
	const char *path_check,	// Prsente uniquement les rvision
							// dj dans path_check si != NULL
	int lookup[])			// Correspondance entre le X'ieme item
							// de la PAGE_PICK et un VERSION_ONE
{
	int ret = -1;
	char *path_rev[200];
	int nbprev = -1;
	if (path_check != NULL){
		nbprev = projet_getrevdir (path_check,path_rev,200);
	}
	if (list.getnb() == 0 || nbprev == 0){
		xconf_error (MSG_U(E_NOVERSION,"Aucune rvision slectionnable"));
	}else{
		ptselitem = selitem;
		int nb = 0;
		setintgstatus();
		if (nbprev > 0){
			for (int i=0; i<nbprev; i++){
				char revstr[MAXSIZ_REVISION];
				revision_path2rev(path_rev[i],revstr);
				REVISION rev (revstr);
				int noitem;
				VERSION_ONE *one = get(&rev,noitem);
				if(one != NULL){
					version_add2dia (one,dia);
					lookup[nb] = noitem;
					nb++;
				}
			}
		}else{
			for (int i=0; i<list.getnb(); i++){
				VERSION_ONE *one = list.getitem(i);
				REVISION_TYPE type = one->rev.type();
				if (type >= minsel && type <= maxsel){
					version_add2dia (one,dia);
					lookup[nb] = i;
					nb++;
				}
			}
		}
		#ifdef MISSING
			ret = version_setpos (pick,nb);
		#else
			ret = 0;
		#endif
	}
	return ret;
}
/*
	Slection interactive d'une rvision d'un projet
	Retourne NULL si abandon ou erreur.
*/
PUBLIC VERSION_ONE *VERSION_DAT::select(
	REVISION_TYPE minsel,	// Slection de rvision ou d'une version.
	REVISION_TYPE maxsel,	//
	const char *path_check)	// Prsente uniquement les rvision
							// dj dans path_check si != NULL
{
	VERSION_ONE *ret = NULL;
	DIALOG_RECORDS dia;
	dia.newf_head ("",MSG_U(H_VERSIONS,"Version\tProg\tDate\tRvision"));
	unsigned selitem = list.getnb()-1;
	int lookup[list.getnb()];
	if (lookup != NULL
		&& setselect (dia,minsel,maxsel,&selitem,path_check,lookup)!=-1){
		int nof = dia.getnb()-1;
		while (1){
			MENU_STATUS code = dia.editmenu(MSG_U(T_VERSIONS,"Versions")
				,""
				,help_nil
				,nof,0);
			if (code == MENU_QUIT || code == MENU_ESCAPE){
				break;
			}else{
				VERSION_ONE *one = list.getitem(lookup[nof]);
				UISTATE state;
				diajava_lastmousestate (state);
				if (state.rightb){
					one->showinfo();
				}else{
					ret = one;
					break;
				}
			}
		}
	}
	return ret;
}
/*
	Slection d'une version comme version courante pour usager
	Retourne != 0 si slection valide.
*/
int version_setmain (
	USERINFO *user,
	const char *path_check)	// Prsente uniquement les rvision
							// dj dans path_check si != NULL
{
	int ret = 0;
	VERSION_DAT version(user);
	VERSION_ONE *rev = version.select (REVISION_VERSION
		,REVISION_LIVRE,path_check);
	if (rev != NULL && user->setversion (rev)){
		// Vrifie si l'environnement existe pour l'usager
		ret = 1;
	}
	return ret;
}

/*
	Enregistre toutes les rvisions dans un page (de slection)
*/
PUBLIC void VERSION_DAT::setuppick(DIALOG &dia)
{
	int nb = list.getnb();
	setintgstatus();
	for (int i=0; i<nb; i++){
		VERSION_ONE *one = list.getitem(i);
		version_add2dia (one,dia);
	}
	#ifdef MISSING
		version_setpos (&sp,nb);
	#endif
}
